#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>




/** brief Main routine */

/* CAUTION :
*           unsigned int : 16bits in ATmega
*           uint32_t     : 32bits in ATmega
*/

static int usart1_tx(char c, FILE *stream)
{
	if(c == '\n')
	{
		while ((USART1.STATUS & USART_DREIF_bm) == 0);
		USART1.TXDATAL = 0x0D;	// carriage return
	}

	while ((USART1.STATUS & USART_DREIF_bm) == 0);
	USART1.TXDATAL = c;

	while ((USART1.STATUS & USART_TXCIF_bm) == 1);


	return 0;
}




static void usart1_rx(FILE *stream)
{
	char data;

	while ((USART1.STATUS & USART_RXCIF_bm) == 0);

	data = USART1.RXDATAL;

	// Just for test...
	//usart1_tx(data, stream);
}



static void usart1_rx_intr_enable(void)
{
	USART1.CTRLA |= USART_RXCIE_bm;
}



ISR(USART1_RXC_vect)
{
	unsigned char rcv_data;

	while ((USART1.STATUS & USART_RXCIF_bm) == 0);

	rcv_data = USART1.RXDATAL;

	// Just for test...
	//uart1_tx(rcv_data, NULL);
}




/* When you use ANCI-C key input API... */
static FILE avr_stdio = FDEV_SETUP_STREAM(usart1_tx, usart1_rx, _FDEV_SETUP_RW);

//USART baud rate calculation
#define CLK_PER (F_CPU)
#define USART_BAUD_RATE(F_BAUD) ((float)CLK_PER * 64 / (16 * (float)F_BAUD))



void USART1_Printf_Init(uint32_t baudrate)
{
	PORTC.DIRSET |= (1 << PIN0_bp);//PB2 as output
	PORTC.OUTCLR |= (1 << PIN0_bp);// initial level false
	
	// USART baud rate initialization as mentioned in the datasheet
	
	int8_t sigrow_value = SIGROW.OSC20ERR3V;				// Read signed error

	baudrate *= (1024 + sigrow_value);							// Sum resolution + Error
	baudrate /=  1024;											// Divide by resolution

	USART1.BAUD = (uint16_t)USART_BAUD_RATE(baudrate);			// Set adjusted baud rate

	USART1.CTRLB = (1 << USART_RXEN_bp) | (1 << USART_TXEN_bp) | (1 << USART_SFDEN_bp);	// Transmitter enable (normal mode)

	// Assign device(avr_stdio) to stdout & stdin.
	stdout = stdin = &avr_stdio;

	_delay_ms(10);
}
